@@ -1,11 +1,10 @@
 #' Plot estimated and fitted model parameters
 #'
 #' @param vst_out The output of a vst run
+#' @param xaxis Variable to plot on X axis; default is "gmean"
 #' @param show_theta Whether to show the theta parameter; default is FALSE (only the overdispersion factor is shown)
 #' @param show_var Whether to show the average model variance; default is FALSE
-#' @param verbosity An integer specifying whether to show only messages (1), messages and progress bars (2) or nothing (0) while the function is running; default is 2
-#' @param verbose Deprecated; use verbosity instead
-#' @param show_progress Deprecated; use verbosity instead
+#' @param verbosity An integer specifying the verbosity level: 0 (silent, no messages), 1 (show messages only), or 2 (show messages and progress bars); default is 2
 #'
 #' @return A ggplot object
 #'
@@ -20,22 +19,8 @@
 #' plot_model_pars(vst_out)
 #' }
 #'
-plot_model_pars <- function(vst_out, show_theta = FALSE, show_var = FALSE,
-                            verbosity = 2, verbose = NULL, show_progress = NULL) {
-  # Take care of deprecated arguments
-  if (!is.null(verbose)) {
-    warning("The 'verbose' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
-    verbosity <- as.numeric(verbose)
-  }
-  if (!is.null(show_progress)) {
-    warning("The 'show_progress' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
-    if (show_progress) {
-      verbosity <- 2
-    } else {
-      verbosity <- min(verbosity, 1)
-    }
-  }
-
+plot_model_pars <- function(vst_out, xaxis="gmean", show_theta = FALSE, show_var = FALSE,
+                            verbosity = 2) {
   if (! 'gmean' %in% names(vst_out$gene_attr)) {
     stop('vst_out must contain a data frame named gene_attr with a column named gmean (perhaps call vst with return_gene_attr = TRUE)')
   }
@@ -62,20 +47,38 @@ plot_model_pars <- function(vst_out, show_theta = FALSE, show_var = FALSE,
   df$parameter <- factor(df$parameter, levels = ordered_par_names)
   df_fit <- melt(mp_fit, varnames = c('gene', 'parameter'), as.is = TRUE)
   df_fit$parameter <- factor(df_fit$parameter, levels = ordered_par_names)
-  df$gene_gmean <- vst_out$gene_attr[df$gene, 'gmean']
   df$is_outl <- vst_out$model_pars_outliers
-  df_fit$gene_gmean <- vst_out$gene_attr[df_fit$gene, 'gmean']
   df$type <- 'single gene estimate'
   df_fit$type <- 'regularized'
   df_fit$is_outl <- FALSE
+
+  if (startsWith(x = vst_out$arguments$method, prefix = 'offset') | (xaxis=="amean")) {
+    df$x <- vst_out$gene_attr[df$gene, 'amean']
+    df_fit$x <- vst_out$gene_attr[df_fit$gene, 'amean']
+    xlab <- 'Arithmetic mean of gene [log10]'
+  } else {
+    df$x <- vst_out$gene_attr[df$gene, 'gmean']
+    df_fit$x <- vst_out$gene_attr[df_fit$gene, 'gmean']
+    xlab <- 'Geometric mean of gene [log10]'
+  }
+
   df_plot <- rbind(df, df_fit)
   df_plot$parameter <- factor(df_plot$parameter, levels = ordered_par_names)
-  g <- ggplot(df_plot, aes_(x=~log10(gene_gmean), y=~value, color=~type)) +
+
+  if (!vst_out$arguments$do_regularize || startsWith(x = vst_out$arguments$method, prefix = 'offset')) {
+    df_plot <- df_plot[df_plot$type == 'single gene estimate', ]
+    legend_pos <- 'none'
+  } else {
+    legend_pos <- 'bottom'
+  }
+
+  g <- ggplot(df_plot, aes_(x=~log10(x), y=~value, color=~type)) +
     geom_point(data=df, aes_(shape=~is_outl), size=0.5, alpha=0.5) +
     scale_shape_manual(values=c(16, 4), guide = FALSE) +
     geom_point(data=df_fit, size=0.66, alpha=0.5, shape=16) +
     facet_wrap(~ parameter, scales = 'free_y', ncol = ncol(mp)) +
-    theme(legend.position='bottom')
+    theme(legend.position = legend_pos) +
+    xlab(label = xlab)
   return(g)
 }
 
@@ -84,7 +87,12 @@ get_model_par_mat <- function(vst_out, model_pars, use_nonreg, show_theta = FALS
                               verbosity = 2) {
   mp <- model_pars
   # transform theta to overdispersion factor
-  mp[, 1] <- log10(1 + vst_out$gene_attr[rownames(mp), 'gmean'] / mp[, 'theta'])
+  if (startsWith(x = vst_out$arguments$method, prefix = 'offset')) {
+    mp[, 1] <- log10(1 + vst_out$gene_attr[rownames(mp), 'amean'] / mp[, 'theta'])
+  } else {
+    mp[, 1] <- log10(1 + vst_out$gene_attr[rownames(mp), 'gmean'] / mp[, 'theta'])
+  }
+
   colnames(mp)[1] <- 'log10(od_factor)'
   ordered_par_names <- colnames(mp)[c(2:ncol(mp), 1)]
   if (show_theta) {
@@ -118,7 +126,7 @@ get_nb_fit <- function(x, umi, gene, cell_attr, as_poisson = FALSE) {
   res <- pmin(res, x$arguments$res_clip_range[2])
   res <- pmax(res, x$arguments$res_clip_range[1])
   ret_df <- data.frame(mu = mu, sd = sd, res = res)
-  # in case we have individaul (non-regularized) parameters
+  # in case we have individual (non-regularized) parameters
   if (gene %in% rownames(x$model_pars)) {
     coefs <- x$model_pars[gene, -1, drop=FALSE]
     theta <- x$model_pars[gene, 1]
@@ -157,6 +165,7 @@ get_nb_fit <- function(x, umi, gene, cell_attr, as_poisson = FALSE) {
 #' @import ggplot2
 #' @import reshape2
 #' @importFrom gridExtra grid.arrange
+#' @importFrom stats bw.nrd0
 #'
 #' @export
 #'
@@ -168,8 +177,8 @@ get_nb_fit <- function(x, umi, gene, cell_attr, as_poisson = FALSE) {
 #'
 plot_model <- function(x, umi, goi, x_var = x$arguments$latent_var[1], cell_attr = x$cell_attr,
                        do_log = TRUE, show_fit = TRUE, show_nr = FALSE, plot_residual = FALSE,
-                       batches = NULL, as_poisson = FALSE, arrange_vertical = TRUE, show_density = TRUE,
-                       gg_cmds = NULL) {
+                       batches = NULL, as_poisson = FALSE, arrange_vertical = TRUE,
+                       show_density = FALSE, gg_cmds = NULL) {
   if (is.null(batches)) {
     if (!is.null(x$arguments$batch_var)) {
       batches <- cell_attr[, x$arguments$batch_var]
@@ -203,7 +212,9 @@ plot_model <- function(x, umi, goi, x_var = x$arguments$latent_var[1], cell_attr
   df$gene <- factor(df$gene, ordered=TRUE, levels=unique(df$gene))
   g <- ggplot(df, aes_(~x, ~y)) + geom_point(alpha=0.5, shape=16)
   if (show_density) {
-    g <- g + geom_density_2d(color = 'lightblue', size=0.5)
+    bandwidths <- c(bw.nrd0(g$data$x), bw.nrd0(g$data$y))
+    g <- g + geom_density_2d(color = 'lightblue', size=0.5, h = bandwidths,
+                             contour_var = "ndensity")
   }
   if (show_fit) {
     for (b in unique(df$batch)) {
